#######################################################################
# searchViewAndSave.praat (Kyuchul Yoon, kyoon@ynu.ac.kr)
# v1: Identifies target phonemes/words with respect to a boundary
# v2: Added option for automatic saving
# v3: sound/label objects being removed while searching. Removed allophone tier processing
# v4: search for romanized tiers too
#######################################################################

form Search Parameters
	word labelFolder_(TextGrid) label
	word soundFolder_(wav) sound
	comment * Choose target type, age and gender
	optionmenu TargetType: 1
		option Orthographic pWords
		option Orthographic pWords - romanized
		option Pronounced pWords
		option Pronounced pWords - romanized
		option Phonemes
	optionmenu TargetAge: 5
		option 10s
		option 20s
		option 30s
		option 40s
		option any
	optionmenu TargetGender: 3
		option male
		option female
		option any
	comment * Search string with respect to a boundary
	word leftTarget_(*_for_anything) 학교에
	word rightTarget_(*_for_anything) *
	comment * Number of search results
	integer numResults_(0_for_no_limit) 10
	comment * Window size and output files
	real windowSize_(in_sec) 3.5
	word logFile_(to_be_created) LOGFILE.txt
	word outFolder_(to_be_created) extractedFolder
endform

system_nocheck mkdir 'outFolder$'
yourChoice = 1

# orthographic/pronounced pWords tier, phonemes tier
halfWindow = windowSize / 2
phoneTier = 1
#orthoTier = 4
#proTier = 2

# With romanized textgrids
orthoTier = 5
orthoTierRom = 6
proTier = 2
proTierRom = 3

# Check user input for search tier
if targetType = 1
	targetTier = orthoTier
elsif targetType = 2
	targetTier = orthoTierRom
elsif targetType = 3
	targetTier = proTier
elsif targetType = 4
	targetTier = proTierRom
elsif targetType = 5
	targetTier = phoneTier
endif

# Read the list of label files in the label folder
Create Strings as file list... fileListObj 'labelFolder$'/*.TextGrid
Sort
numFiles = Get number of strings
pause 'numFiles' TextGrids identified. Continue?

# Check user input for output in log file
if targetType = 1
	targetType$ = "orthographic pWords"
elsif targetType = 2
	targetType$ = "orthographic pWords romanized"
elsif targetType = 3
	targetType$ = "pronounced pWords"
elsif targetType = 4
	targetType$ = "pronounced pWords romanized"
elsif targetType = 5
	targetType$ = "phonemes"
endif

if targetAge = 1
	targetAge$ = "10s"
elsif targetAge = 2
	targetAge$ = "20s"
elsif targetAge = 3
	targetAge$ = "30s"
elsif targetAge = 4
	targetAge$ = "40s"
elsif targetAge = 5
	targetAge$ = "any"
endif

if targetGender = 1
	targetGender$ = "male"
elsif targetGender = 2
	targetGender$ = "female"
elsif targetGender = 3
	targetGender$ = "any"
endif


fileappend 'logFile$' wrt/ a boundary, left search string:'tab$''leftTarget$'
...'tab$'right search string:'tab$''rightTarget$''tab$'number of query results:
...'tab$''numResults''tab$'targetType:'tab$''targetType$''tab$'targetAge:'tab$'
...'targetAge$''tab$'targetGender:'tab$''targetGender$''newline$'
fileappend 'logFile$' filename'tab$'PrevInterval'tab$'NextInterval'tab$'
...PrevIntText'tab$'NextIntText'tab$'BoundaryTime'newline$'

# Loop through each file
countNumResults = 0
for iFile to numFiles
	select Strings fileListObj
	fileName$ = Get string... iFile
	fileNo$ = mid$(fileName$,2,2)
	gender$ = mid$(fileName$,4,1)
	fileNo = 'fileNo$'

	# Check for target files
	if targetAge = 1
		if targetGender = 1
			if fileNo <= 10 and gender$ = "m"
				call identifyTarget 'fileName$'
			endif
		elsif targetGender = 2
			if fileNo <= 10 and gender$ = "f"
				call identifyTarget 'fileName$'
			endif
		elsif targetGender = 3
			if fileNo <= 10
				call identifyTarget 'fileName$'
			endif
		endif
	elsif targetAge = 2
		if targetGender = 1
			if fileNo <= 20 and fileNo > 10 and gender$ = "m"
				call identifyTarget 'fileName$'
			endif
		elsif targetGender = 2
			if fileNo <= 20 and fileNo > 10 and gender$ = "f"
				call identifyTarget 'fileName$'
			endif
		elsif targetGender = 3
			if fileNo <= 20 and fileNo > 10
				call identifyTarget 'fileName$'
			endif
		endif
	elsif targetAge = 3
		if targetGender = 1
			if fileNo <= 30 and fileNo > 20 and gender$ = "m"
				call identifyTarget 'fileName$'
			endif
		elsif targetGender = 2
			if fileNo <= 30 and fileNo > 20 and gender$ = "f"
				call identifyTarget 'fileName$'
			endif
		elsif targetGender = 3
			if fileNo <= 30 and fileNo > 20
				call identifyTarget 'fileName$'
			endif
		endif
	elsif targetAge = 4
		if targetGender = 1
			if fileNo <= 40 and fileNo > 30 and gender$ = "m"
				call identifyTarget 'fileName$'
			endif
		elsif targetGender = 2
			if fileNo <= 40 and fileNo > 30 and gender$ = "f"
				call identifyTarget 'fileName$'
			endif
		elsif targetGender = 3
			if fileNo <= 40 and fileNo > 30
				call identifyTarget 'fileName$'
			endif
		endif
	elsif targetAge = 5
		if targetGender = 1
			if gender$ = "m"
				call identifyTarget 'fileName$'
			endif
		elsif targetGender = 2
			if gender$ = "f"
				call identifyTarget 'fileName$'
			endif
		elsif targetGender = 3
			call identifyTarget 'fileName$'
		endif
	endif
endfor

# Procedure for identifying target segments
procedure identifyTarget filename$
	file$ = left$(filename$,8)
	soundname$ = file$ + ".wav"
	Read from file... 'soundFolder$'/'soundname$'

	Read from file... 'labelFolder$'/'filename$'
	prefix$ = filename$ - ".TextGrid"
	numIntervals = Get number of intervals... targetTier

	for iInterval from 2 to numIntervals
		select TextGrid 'prefix$'
		rightIntervalText$ = Get label of interval... targetTier iInterval
		iPrevInterval = iInterval - 1
		leftIntervalText$ = Get label of interval... targetTier (iInterval-1)
		
		# Query starts. Check the four logically possible cases.
		# Switch the flags if necessary and get the sample letters.
		leftFlag = 0
		rightFlag = 0
		if (leftTarget$ <> "*" and rightTarget$ <> "*")
			leftFlag = 1
			rightFlag = 1
			numLeftTarget = length(leftTarget$)
			numRightTarget = length(rightTarget$)
			leftSample$ = right$(leftIntervalText$,numLeftTarget)
			rightSample$ = left$(rightIntervalText$,numRightTarget)
		elsif (leftTarget$ <> "*" and rightTarget$ = "*")
			leftFlag = 1
			numLeftTarget = length(leftTarget$)
			leftSample$ = right$(leftIntervalText$,numLeftTarget)
		elsif (rightTarget$ <> "*" and leftTarget$ = "*")
			rightFlag = 1
			numRightTarget = length(rightTarget$)
			rightSample$ = left$(rightIntervalText$,numRightTarget)
		else
			exit Wrong search strings! Both *
		endif

		# Get the sample letters and compare with the target letters.
		# If both flags are positive, then it means we've got letters on both sides.
		# If only the left flag is positive, then the right side is * (anything).
		# If only the right flag is positive, then the left side is * (anything).
		if (leftFlag = 1 and rightFlag = 1)
			if (leftSample$ = leftTarget$ and rightSample$ = rightTarget$)
				countNumResults = countNumResults + 1
				boundaryTime = Get starting point... targetTier iInterval
				call showSave
				fileappend 'logFile$' 'filename$''tab$''iPrevInterval''tab$''iInterval''tab$'
				...'leftIntervalText$''tab$''rightIntervalText$''tab$''boundaryTime:2''newline$'
			endif
		elsif leftFlag = 1
			if leftSample$ = leftTarget$
				countNumResults = countNumResults + 1
				boundaryTime = Get starting point... targetTier iInterval
				call showSave
				fileappend 'logFile$' 'filename$''tab$''iPrevInterval''tab$''iInterval''tab$'
				...'leftIntervalText$''tab$''rightIntervalText$''tab$''boundaryTime:2''newline$'
			endif
		elsif rightFlag = 1
			if rightSample$ = rightTarget$
				countNumResults = countNumResults + 1
				boundaryTime = Get starting point... targetTier iInterval
				call showSave
				fileappend 'logFile$' 'filename$''tab$''iPrevInterval''tab$''iInterval''tab$'
				...'leftIntervalText$''tab$''rightIntervalText$''tab$''boundaryTime:2''newline$'
			endif
		endif

		# Check the count of search results
		if numResults = 0
		elsif countNumResults = numResults
			exit Number of query results reached!
		endif
	endfor
	plus Sound 'file$'
	Remove
endproc

# Procedure for showing and saving target visible parts
procedure showSave
	# Show the window
	select Sound 'file$'
	plus TextGrid 'prefix$'

	if yourChoice <> 3
		Edit
		editor TextGrid 'prefix$'
			Select... (boundaryTime-halfWindow) (boundaryTime+halfWindow)
			Zoom to selection
			Move cursor to... boundaryTime
			beginPause("Check to save visible part")
				comment("Save the visible part?")
				choice("yourChoice",1)
					option("Yes")
					option("No")
					option("Automatic")
			clicked = endPause("Continue", 1)
			Close
		endeditor
	endif

	if (yourChoice = 1 or yourChoice = 3)
		extSound$ = "'iInterval'" + "-" + soundname$
		extTextGrid$ = "'iInterval'" + "-" + filename$
		select Sound 'file$'
		Extract part... (boundaryTime-halfWindow) (boundaryTime+halfWindow) Rectangular 1 no
		Write to WAV file... 'outFolder$'/'extSound$'
		Remove
		select TextGrid 'prefix$'
		Extract part... (boundaryTime-halfWindow) (boundaryTime+halfWindow) Rectangular no
		Write to text file... 'outFolder$'/'extTextGrid$'
		Remove
	endif
endproc
############ END OF SCRIPT ###############